
'use strict';

module.exports = (ctx, next) => {
	const statusCode = ctx.status;
	const msg = ctx.message;

	ctx.app.logger.debug(`Status code: ${statusCode}`);

	if (statusCode >= 300 && statusCode < 400) {
		if (!('followRedirect' in ctx.app.config.downloader)) {
			const e = new Error(`Status Code: ${statusCode}, ${msg}. not has property followRedirect`);
			e.code = statusCode;
			throw e;
		} else {
			if (ctx.app.config.downloader.followRedirect) {
				const e = new Error(`Status Code: ${statusCode}, ${msg}. not allowed to follow redirect`);
				e.code = statusCode;
				throw e;
			}
			
			return next();
		}
	} else if (statusCode >= 400) {
		const body = ctx.body;
		let errMsg;
		
		if (Buffer.isBuffer(body)) {
			if (body.length === 0) {
				errMsg = `Status Code: ${statusCode}, ${msg}. response body empty`;
			} else {
				errMsg = `Status Code: ${statusCode}, ${msg}. response body: ${body.toString()}`;
			}
		} else {
			errMsg = `Status Code: ${statusCode}, ${msg}. response body undefined`;
		}
		
		const e = new Error(errMsg);
		e.code = statusCode;
		throw e;
	}

	return next();
};
