
'use strict';

const expect = require('chai').expect;
const Mof = require('mof');
const statuscode = require('../');
const sinon = require('sinon');

describe('mof-statuscode ', () => {
	let app;
	beforeEach(() => {
		app = new Mof();
		app.logger = {debug: function(){}, info:function(){}, error:function(){}};
		app.config= {downloader: {} } ;
	});

	it('should catch 4XX / 5XX with filled buffered response body', done => {
		let context = { status: 400, body: Buffer.from('test'), app, message:'this is error message'};

		app.use(statuscode);

		let cb = sinon.spy();

		app.callback(cb, e => {
			expect(cb.called).to.be.false;
			expect(e.message).to.be.eql(`Status Code: ${context.status}, ${context.message}. response body: ${context.body}`);
			expect(e.code).to.eql(context.status);
			done();
		})(context);
	});

	it('should catch 4XX / 5XX with empty buffered response body', done => {
		let context = { status: 500, body: Buffer.from(''), app, message:'this is error message'};

		app.use(statuscode);

		let cb = sinon.spy();

		app.callback(cb, e => {
			expect(cb.called).to.be.false;
			expect(e.message).to.eql(`Status Code: ${context.status}, ${context.message}. response body empty`);
			expect(e.code).to.eql(context.status);
			done();
		})(context);
	});

	it('should catch 4XX / 5XX with unbuffered response body', done => {
		let context = { status: 400, body: 'test', app, message:'this is error message'};

		app.use(statuscode);

		let cb = sinon.spy();

		app.callback(cb, e => {
			expect(cb.called).to.be.false;
			expect(e.message).to.eql(`Status Code: ${context.status}, ${context.message}. response body undefined`);
			expect(e.code).to.eql(context.status);
			done();
		})(context);
	});

	it('should catch 3XX when not has property followRedirect', done => {
		let context = { status: 300, body: Buffer.from('test'), app , message:'this is error message'};

		app.use(statuscode);

		let cb = sinon.spy();

		app.callback(cb, e => {
			expect(cb.called).to.be.false;
			expect(e.message).to.eql(`Status Code: ${context.status}, ${context.message}. not has property followRedirect`);
			expect(e.code).to.eql(context.status);
			done();
		})(context);
	});

	it('should catch 3XX when has property followRedirect and allow to follow redirect', done => {
		app.config.downloader.followRedirect = true;
		let context = { status: 300, body: Buffer.from('test'), app, message:'this is error message'};

		app.use(statuscode);

		let cb = sinon.spy();

		app.callback(cb, e => {
			expect(cb.called).to.be.false;
			expect(e.message).to.eql(`Status Code: ${context.status}, ${context.message}. not allowed to follow redirect`);
			expect(e.code).to.eql(context.status);
			done();
		})(context);
	});

	it('should pass 3XX when has property followRedirect and not allow to follow redirect', done => {
		app.config.downloader.followRedirect = false;
		let context = { status: 300, body: Buffer.from('test'), app, message:'this is error message'};

		app.use(statuscode);

		let cb = sinon.spy();
		app.use(cb);
		app.callback(() => {
			expect(cb.called).to.be.true;
			done();
		})(context);
	});

	it('should pass 2XX', done => {
		let context = { status: 200, body: Buffer.from('test') , app};

		app.use(statuscode);

		let cb = sinon.spy();
		app.use(cb);
		app.callback(() => {
			expect(cb.called).to.be.true;
			done();
		})(context);
	});
});
